/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.application;

import java.awt.*;
import java.awt.event.*;

import mindbright.ssh.*;
import mindbright.terminal.*;

public class MindTerm {

  Frame       frame;
  MenuBar     menubar;
  TerminalWin term;
  SSHClient   sshClient;

  public static void main(String[] argv) {
    MindTerm controller = new MindTerm();
    controller.startup(argv);
  }

  public void startup(String[] argv) {
    try {
      String  server;
      String  username = null;
      String  cipher   = null;
      boolean haveGUI  = true;
      int     port     = -1;
      int     i;

      SSHStdIO console = new SSHStdIO();
      sshClient        = new SSHClient();
      console.setClient(sshClient);
      sshClient.setConsole(console);

      for(i = 0; i < argv.length; i++) {
	String arg = argv[i];
	if(arg.charAt(0) != '-')
	  break;
	switch(arg.charAt(1)) {
	case 'l':
	  username = argv[++i];
	  break;
	case 'i':
	  break;
	case 'd':
	  haveGUI = false;
	  break;
	case 'V':
	  System.out.println("SSH version " + SSH.VER_SSH + ", protocol version " +
			     SSH.VER_MAJOR + "." + SSH.VER_MINOR);
	  System.exit(0);
	  break;
	case 'c':
	  cipher = argv[++i];
	  if(!cipher.equals("blowfish")) {
	    System.out.println("** Sorry, only blowfish-cipher is supported at this time.");
	    throw new Exception();
	  }
	  break;
	case 'p':
	  port = Integer.parseInt(argv[++i]);
	  break;
	case 'L':
	  sshClient.addLocalPortForward(argv[++i]);
	  break;
	case 'R':
	  sshClient.addRemotePortForward(argv[++i]);
	  break;
	default:
	  throw new Exception();
	}
      }

      if(i < argv.length) {
	server = argv[argv.length - 1];
	sshClient.setServerName(server);
      }

      if(port > 0)
	sshClient.setPort(port);

      // Fetch username on local system if empty
      //
      if(username == null)
	username = System.getProperty("user.name");
      if(username != null)
	sshClient.setUsername(username);

      Thread clientThread = new Thread(sshClient);
      if(haveGUI) {
	initGUI();
	console.setTerminal(term);
	console.setOwnerFrame(frame);
	console.setOwnerName("MindTerm v0.9b");
	console.updateTitle();
      }
      clientThread.start();
      clientThread.join();
      if(frame != null)
	frame.dispose();
      System.exit(0);

    } catch(Exception e) {
      System.out.println("usage: MindTerm [options] [<server>]");
      System.out.println("Options:");
      System.out.println("  -l user     Log in using this user name.");
      System.out.println("  -d          No terminal-window, only dumb command-line and port-forwarding.");
      System.out.println("  -V          Display version number only.");
      System.out.println("  -c cipher   Select encryption algorithm: ``idea'', ``blowfish'', ``3des''");
      System.out.println("  -p port     Connect to this port.  Server must be on the same port.");
      System.out.println("  -L [/plugin/]listen-port:host:port Forward local port to remote address.");
      System.out.println("  -R [/plugin/]listen-port:host:port Forward remote port to local address.");
    }
  }

  public void initGUI() {
    term  = new TerminalWin(80, 24, new TerminalVT100());

    // !!!
    term.setSaveLines(1024);

    frame    = new Frame();
    frame.setLayout(new BorderLayout());


    // !!! Weird, on Linux if the menubar is added here before the window is visible the
    // TerminalWin won't get it's events... TODO: check this
    //
    // frame.setMenuBar(MenuHandler.createMenus());

    frame.add(term, "Center");
    frame.add(term.createScrollbar(), "East");

    frame.pack();
    frame.show();
  }

}

