/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.ssh;

public abstract class SSHChannel extends Thread {

  protected int                channelId;
  protected SSHChannelListener listener;

  public SSHChannel(int channelId) {
    super();
    this.channelId = channelId;
    this.listener  = null;
  }

  public void setSSHChannelListener(SSHChannelListener listener) {
    this.listener = listener;
  }

  public int getId() {
    return channelId;
  }

  public abstract void serviceLoop() throws Exception;

  public void close() {
  }

  public void run() {
    try {
      serviceLoop();
    } catch (Exception e) {
      close();
      if(listener != null)
	listener.close(this);
      /* !!! DEBUG !!!
	 System.out.println("Channel terminated " + channelId);
	 System.out.println("Cause: " + e.toString());
	 if(this instanceof SSHTxChannel)
	 System.out.println("I was a tx-chan");
	 else
	 System.out.println("I was a rx-chan");
	 } catch (ThreadDeath death) {
	 System.out.println("Channel killed " + channelId);
      */
    }
  }

}
