/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.ssh;

import java.io.*;

public class SSHTxChannel extends SSHChannel {

  protected OutputStream out;
  protected SSHPduQueue  queue;

  boolean closePending;

  public SSHTxChannel(OutputStream out, int channelId) {
    super(channelId);
    this.out          = out;
    this.closePending = false;
    queue = new SSHPduQueue(this);
  }

  public SSHPduQueue getQueue() {
    return queue;
  }

  public void setClosePending() {
    closePending = true;
    queue.release();
  }

  public synchronized boolean isClosePending() {
    return closePending;
  }

  public void serviceLoop() throws Exception {
    System.out.println("Starting tx-chan: " + channelId);
    for(;;) {
      SSHPdu pdu;
      // !!! the thread is (hopefully) suspended when we set closePending
      // so we don't have to access a lock each loop
      if(closePending && queue.isEmpty())
	throw new Exception("CLOSE");
      pdu = queue.getFirst();
      //      pdu = pdu.preProcess();
      pdu.writeTo(out);
      //      pdu = pdu.postProcess();
    }
  }

}
